//=============================================================================
// Copyright  Micromega Corporation 2005-2008. All rights reserved.
//
// @file	calltest.c
// @target	Microchip dsPIC C compiler
//
// @author	Cam Thompson, Micromega Corporation
// @version
//	August 26, 2008
//	- original version
//=============================================================================

// select device and set configuration bits
#include <p30f3012.h>
_FOSC(CSW_FSCM_OFF & FRC_PLL16);
_FWDT(WDT_OFF);
_FICD(ICS_PGD3);

// set clock frequency
#define FCY 29480000UL				// clock frequency: 29.48 MHz
#include <libpic30.h>

#include <stdio.h>
#include "fpu.h"

union {                         	// temporary storage
	float fval;
	long  lval;
	unsigned int  ival[2];
	unsigned char bval[4];
} u2;

//------------------ main -----------------------------------------------------

int main(void) {
	unsigned char bval;
	int wval;
	char sbuf[128];

	// UART setup
	__C30_UART = 1;				// select UART1 for stdio
	U1BRG = 95;					// 19,200 baud @ 29.48 MHz
	U1MODEbits.ALTIO = 1;		// alternate I/O port
	U1MODEbits.UARTEN = 1;		// enable UART

	// print program name
	printf("\r\n\r\nCall Test\r\n");

	if (fpu_reset() == SYNC_CHAR) {
		print_version();
		print_CRLF();
		print_CRLF();
	}
	else {
		printf("uM-FPU not detected");
		while(1) ;  			// stop if FPU reset fails
	}

	printf("\r\nfpu_write");
	fpu_write4(SELECTA, 1, LOADPI, FSET0);
	fpu_write3(COPY, 1, 2);
	fpu_write2(SELECTA, 1);
	fpu_write(CLRA);

	fpu_write3(SELECTA, 10, LOADWORD);
	fpu_writeWord(1000);
	fpu_write(FSET0);

	fpu_write2(LWRITE, 11);	
	fpu_writeLong(0x12345678);

	fpu_write2(FWRITE, 12);
	fpu_writeFloat(1000.25);

	fpu_write3(SELECTA, 13, ATOF);
	fpu_writeString("-123.4567");
	fpu_write(FSET0);

	printf("\r\nfpu_read");
	fpu_write2(SELECTA, 11);
	fpu_wait();
	fpu_write(LREADBYTE);
	fpu_readDelay();
	bval = fpu_read();
	printf("\r\nbyte = %02x", bval);

	fpu_write2(SELECTA, 11);
	fpu_wait();
	fpu_write(LREADWORD);
	fpu_readDelay();
	wval = fpu_readWord();
	printf("\r\nword = %02x", wval);

	fpu_wait();
	fpu_write2(LREAD, 11);
	fpu_readDelay();
	u2.lval = fpu_readLong();
	printf("\r\nlong = %8lx", u2.lval);

	fpu_wait();
	fpu_write2(FREAD, 13);
	fpu_readDelay();
	u2.fval = fpu_readFloat();
	printf("\r\nfloat = %8lx, %f", u2.lval, (double)u2.fval);

	fpu_write(FSTATUSA);
	printf("\r\nstatus = %02x", fpu_readStatus());

	printf("\r\nstring = %s", fpu_readString(sbuf));

	printf("\r\nDone.\r\n");
	while(1) ;
}
