//=============================================================================
// Copyright  Micromega Corporation 2005-2008. All rights reserved.
//
// @file	graph.c
// @target	Microchip dsPIC C compiler

//
// This program demonstrates the use of the uM-FPU V3 floating point
// unit with the dsPIC micricontroller. It displays a simple graph of
// the sine and cosine functions on the UART (stdout).
//
// @author	Cam Thompson, Micromega Corporation
// @version
//	August 26, 2008
//	- updated for Microchip dsPIC C compiler
//	February 1, 2008
//	- fixed second graph to be cosine
//	September 20, 2007
//	- original version
//=============================================================================

// select device and set configuration bits
#include <p30f3012.h>
_FOSC(CSW_FSCM_OFF & FRC_PLL16);
_FWDT(WDT_OFF);
_FICD(ICS_PGD3);

// set clock frequency
#define FCY 29480000UL				// clock frequency: 29.48 MHz
#include <libpic30.h>

#include <stdio.h>
#include "fpu.h"

//-------------------- uM-FPU register definitions ----------------------------

#define Angle		1				// angle
#define Point		2				// point on graph

//------------------ local prototypes -----------------------------------------

void graph(void);

//------------------ main -----------------------------------------------------

int main(void) {
	int i;

	// UART setup
	__C30_UART = 1;				// select UART1 for stdio
	U1BRG = 95;					// 19,200 baud @ 29.48 MHz
	U1MODEbits.ALTIO = 1;		// alternate I/O port
	U1MODEbits.UARTEN = 1;		// enable UART

	printf("\r\nGraph demo\r\n");	// program name

	// reset the uM-FPU and print version
	if (fpu_reset() == SYNC_CHAR) {	// reset FPU and check synchronization
		print_version();			// print version
		print_CRLF();
	}
	else {
		printf("uM-FPU not detected");
		return(0);
	}

	// graph sine from 0 to 360 degrees (2*PI radians)
	printf("\r\nGraph of sine\r\n");
	printf("-1        0         1\r\n");
	printf("---------------------\r\n");
    for (i = 0; i <= 20; i++) {
		fpu_write4(SELECTA, Angle, LOADPI, FSET0);
		fpu_write4(FMULI, 2*i, FDIVI, 20);
		fpu_write(SIN);
		graph();
	}

    // graph cosine from 0 to 360 degrees (2*PI radians)
	printf("\r\nGraph of cosine\r\n");
	printf("-1        0         1\r\n");
	printf("---------------------\r\n");
    for (i = 0; i <= 20; i++) {
		fpu_write4(SELECTA, Angle, LOADPI, FSET0);
		fpu_write4(FMULI, 2*i, FDIVI, 20);
		fpu_write(COS);
		graph();
	}

	printf("\r\nDone.\r\n");
	return 0;
}

//-------------------- graph --------------------------------------------------

void graph(void) {
	int i;
	char point, ch;

    // calculate point on graph for value
	fpu_write4(SELECTA, Point, FSET, Angle);
	fpu_write4(FMULI, 10, ROUND, FIX);
	fpu_wait();
	fpu_write(LREADBYTE);
    point = fpu_readByte();

    // plot the point on the graph
	for (i = -10; i <= 10; i++) {
		if (i == point) ch = '*';
		else if (i == 0) ch = '|';
		else ch = ' ';
		putchar(ch);
	}

    // display the angle value
	putchar(' ');
	fpu_write2(SELECTA, Angle);
	print_float(85);
	print_CRLF();
}
